///////////////////////////////////////////////////////////////////////////////
/*! \file Lighting.h
 *  \brief Framework Nemo
 *  \author C. Crassin
 *
 * $Id: MessageHandler.h,v 1.2 2007/09/14 11:54:56 gascuel Exp $
 */
///////////////////////////////////////////////////////////////////////////////
// History:
//
// 2005-01-01 : Creation.

#ifndef MessageHandler_h
#define MessageHandler_h

#include <iostream>
#include <string>
#include <map>
#include <iostream>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

///MODIF DIGICT///

namespace NemoGraphics {
/**  \class MessageHandler
  *  \brief Message handling class. 
  *  \author Cyril Crassin
  *  \version 0.1
  *  \date    09/2007
  *
  *  This class is used for messaged output. Messages have a type and a handler can be set to overwrite deffault stderr output.
  */
class MessageHandler {
	static MessageHandler *instance;

	typedef void (*PtrFctHandler)(int, const char*);
	PtrFctHandler handlerFct[5];

	const static int numTypes=5;

	char buff[4096];
	void message(int type, const char *msg, va_list ap){
	
		if(type>=numTypes)//invalid type
			return;

		vsprintf ( buff, msg, ap );

		if(handlerFct[type]){		
			this->handlerFct[type](type, buff);
		}else{
			switch(type){
				case NEMO_MSG_INFO:
					fprintf( stderr, "INFO: " );
					break;
				case NEMO_MSG_DEBUG:
					fprintf( stderr, "DEBUG: " );
					break;
				case NEMO_MSG_WARNING:
					fprintf( stderr, "WARNING: " );
					break;
				case NEMO_MSG_CRITICAL:
					fprintf( stderr, "CRITICAL ERROR: " );
					break;
				case NEMO_MSG_FATAL:
					fprintf( stderr, "FATAL ERROR: " );
					break;
			}
			fprintf( stderr, buff );
			fprintf( stderr, "\n" );		// add newline

			if(type==NEMO_MSG_CRITICAL || type==NEMO_MSG_FATAL){
				int cc;
				std::cin>>cc;
			}

		}
	}

public:

	MessageHandler(){
		for(int i=0; i<numTypes; i++)
			handlerFct[i]=NULL;
	}

	static MessageHandler &getInstance(){
		if(instance==NULL)
			instance = new MessageHandler();
		return *instance;
	}

	enum MessageType{NEMO_MSG_INFO=0, NEMO_MSG_DEBUG, NEMO_MSG_WARNING, NEMO_MSG_CRITICAL, NEMO_MSG_FATAL};

	void message(int type, const char *msg, ...){
		va_list ap;
		va_start( ap, msg );
		this->message(type, msg, ap);
		va_end( ap );
	}


	void info(const char *msg, ...){
		va_list ap;
		va_start( ap, msg );	
		this->message ( NEMO_MSG_INFO, msg, ap );
		va_end( ap );
	}

	void debug(const char *msg, ...){
		va_list ap;
		va_start( ap, msg );	
		this->message ( NEMO_MSG_DEBUG, msg, ap );
		va_end( ap );
	}

	void warning(const char *msg, ...){
		va_list ap;
		va_start( ap, msg );	
		this->message ( NEMO_MSG_WARNING, msg, ap );
		va_end( ap );
	}

	void critical(const char *msg, ...){
		va_list ap;
		va_start( ap, msg );	
		this->message ( NEMO_MSG_CRITICAL, msg, ap );
		va_end( ap );
	}

	void fatal(const char *msg, ...){
		va_list ap;
		va_start( ap, msg );	
		this->message ( NEMO_MSG_FATAL, msg, ap );
		va_end( ap );
	}


	void attachHandler(PtrFctHandler fct){
		for(int i=0; i<numTypes; i++)
			this->handlerFct[i]=fct;
	}
	void detachHandler(){
		for(int i=0; i<numTypes; i++)
			this->handlerFct[i]=NULL;
	}	

	void attachHandler(int type, PtrFctHandler fct){
		if(type>=numTypes)//invalid type
			return;

		this->handlerFct[type]=fct;
	}
	void detachHandler(int type){
		if(type>=numTypes)//invalid type
			return;

		this->handlerFct[type]=NULL;
	}
};


inline MessageHandler &NEMO_Msg(){
	return MessageHandler::getInstance();
}

inline MessageHandler &NEMO_msg(){ //transition phase !
	return MessageHandler::getInstance();
}
};
#endif
