///////////////////////////////////////////////////////////////////////////////
/*! \file TimerGL.h
 *  \brief Framework Nemo
 *  \author C. Crassin
 *
 */
///////////////////////////////////////////////////////////////////////////////


#ifndef NEMO_TimerGL_h
#define NEMO_TimerGL_h

#include "UtilsGL.h"
#include <map>
#include <string>
#include <iomanip>
#include <iostream>

namespace NemoGraphics {

#define NUM_GL_PERFQUERIES 10

class PerfTimerGL {

	GLuint gltimers[NUM_GL_PERFQUERIES];
	int curGLTimer;
	int maxGLTimerUsed;

	std::string timersMessages[NUM_GL_PERFQUERIES];
	std::map<unsigned long, int> timersMsgMap;


	static int globalState;

public:
	PerfTimerGL(){
		glGenQueries( NUM_GL_PERFQUERIES, gltimers);
		curGLTimer=0;
		maxGLTimerUsed=-1;
	}
	~PerfTimerGL(){
		
	}

	static void globalEnable(){
		globalState=true;
	}
	static void globalDisable(){
		globalState=false;
	}

	int startTimer(const std::string &msg){
		if(globalState){
			timersMsgMap[NemoGraphics::hashString(msg.c_str())]=curGLTimer;
			timersMessages[curGLTimer]=msg;
			glBeginQuery( GL_TIME_ELAPSED_EXT, gltimers[curGLTimer]);
		}
		return curGLTimer;
	}

	void stopTimer(){
		if(globalState){
			glEndQuery( GL_TIME_ELAPSED_EXT);

			if(curGLTimer>maxGLTimerUsed)
				maxGLTimerUsed=curGLTimer;
			curGLTimer=(curGLTimer+1)%NUM_GL_PERFQUERIES;
		}
	}

	double getTime(int timernum){
		if(globalState){
			GLint64EXT queryTime = 0;

			glGetQueryObjecti64vEXT( gltimers[timernum], GL_QUERY_RESULT, &queryTime);

			return queryTime / 1000000000.0; //convert to seconds
		}else{
			return 0.0;
		}
	}

	double getTime(const std::string &msg){
		return this->getTime(timersMsgMap[NemoGraphics::hashString(msg.c_str())]);
	}

	 /*float getLatestTime() {
        int testNum = _activeTimer;
        GLint64EXT elapsedTime = 0;
        GLint queryReady = 0;

        do {
            testNum = ( testNum + 4) % 5;
            glGetQueryObjectiv( _timers[testNum], GL_QUERY_RESULT_AVAILABLE, &queryReady);
        } while (!queryReady);

        glGetQueryObjecti64vEXT( _timers[testNum], GL_QUERY_RESULT, &elapsedTime);

        return elapsedTime / 1000000.0f; //convert to milliseconds
    }*/


	inline void displayTimes(const std::string &msg){
		if(globalState){

			std::cout<<msg<<" :\n";

			for(int i=0; i<=maxGLTimerUsed; i++)
				std::cout<<timersMessages[i]<<" -> "<<std::setw(15) <<getTime(i)<<" , ";
			std::cout<<"\n";
		}
	}

};

};

#endif
